import at.jku.ssw.In;
import at.jku.ssw.Out;
import at.jku.ssw.TextSearcher;
import at.jku.students.BoyerMooreSearcher;
import at.jku.students.BruteForceSearcher;
import java.util.ArrayList;

class TextSearcherTest {

  static final String FILENAME_TEXT = "alice.txt";
//  static final String FILENAME_PATTERNS = "patterns.MacLinux.txt";
  static final String FILENAME_PATTERNS = "patterns.Windows.txt";

  static class Result {

    int cnt = 0;
    long start = 0;
    long end = 0;
    long time = 0;

    void add(Result r) {
      this.cnt += r.cnt;
      this.time += r.time;
    }
  }

  public static void main(String[] args) {
    test("\n*** Test 1 ***", new BruteForceSearcher());
    test("\n*** Test 2 ***", new BoyerMooreSearcher());
    test3();
  }

  static void test(String title, TextSearcher s1) {
    Out.println(title);
    Out.println("countOccurences(\"wettergott\", \"Gott\", false, false)=" + s1.countOccurences("wettergott", "Gott", false, false));
    Out.println("countOccurences(\"wettergott\", \"Gott\", true, false)=" + s1.countOccurences("wettergott", "Gott", true, false));
    Out.println("countOccurences(\"wettergott\", \"tt\", false)=" + s1.countOccurences("wettergott", "tt", false, false));
    Out.println("countOccurences(\"wettergott\", \"t\", false)=" + s1.countOccurences("wettergott", "t", false, false));
    Out.println("countOccurences(\"Er ist Kaiser\", \"er\", false, false)=" + s1.countOccurences("Er ist Kaiser", "er", false, false));
    Out.println("countOccurences(\"Er ist Kaiser\", \"er\", false, true)=" + s1.countOccurences("Er ist Kaiser", "er", false, true));
    Out.println("countOccurences(\"Er ist Kaiser\", \"kaiser\", false, true)=" + s1.countOccurences("Er ist Kaiser", "kaiser", false, true));
    Out.println("countOccurences(\"Er ist Kaiser\", \"er\", true, false)=" + s1.countOccurences("Er ist Kaiser", "er", true, false));
    Out.println("countOccurences(\"Er ist Kaiser\", \"er\", true, true)=" + s1.countOccurences("Er ist Kaiser", "er", true, true));
    Out.println("countOccurences(\"Er ist Kaiser\", \"Kaiser\", true, true)=" + s1.countOccurences("Er ist Kaiser", "Kaiser", true, true));
    Out.println();
    Out.println("countOccurences(\"Annanna\", \"Anna\", false, false)=" + s1.countOccurences("Annanna", "Anna", false, false));
    Out.println("countOccurences(\"Annanna\", \"Anna\", true, false)=" + s1.countOccurences("Annanna", "Anna", true, false));
    Out.println("countOccurences(\"Annanna\", \"Annanna\", false, true)=" + s1.countOccurences("Annanna", "Annanna", false, true));
  }
  
  static void test3() {
    Out.println("\n*** Test 3 ***");
    evaluate(
            readText(FILENAME_TEXT).toLowerCase(),
            // new String[] { "it" } );
            readPatterns(FILENAME_PATTERNS));
  }

  static String readText(String filename) {
    In.open(filename);
    if (!In.done()) {
      throw new IllegalArgumentException("File not found: " + filename);
    }
    String text = In.readFile();
    In.close();
    return text;
  }

  static String[] readPatterns(String filename) {
    In.open(filename);
    if (!In.done()) {
      throw new IllegalArgumentException("File not found: " + filename);
    }
    ArrayList<String> result = new ArrayList<String>();
    do {
      String line = In.readLine().trim();
      if (In.done()) {
        result.add(line);
      }
    } while (In.done());
    In.close();
    return result.toArray(new String[0]);
  }

  static void evaluate(String text, String[] patterns) {
    TextSearcher brute = new BruteForceSearcher();
    TextSearcher boyer = new BoyerMooreSearcher();
    Result total1 = new Result(), total2 = new Result();
    Out.println(String.format("\n%-20s %4s %6s %4s %6s",
            "Pattern", "#", "Brute", "#", "Boyer"));
    Out.println("---------------------------------------------");
    int cnt = 0;
    for (String p : patterns) {
      // if (++cnt == 9) break;
      Result r1 = evaluate(brute, text, p);
      Result r2 = evaluate(boyer, text, p);
      Out.println(String.format("%-20s %4d %6d %4d %6d",
              p, r1.cnt, r1.time, r2.cnt, r2.time));
      total1.add(r1);
      total2.add(r2);
    }
    Out.println("---------------------------------------------");
    Out.println(String.format("%25d %6d %4d %6d",
            total1.cnt, total1.time, total2.cnt, total2.time));
  }

  static Result evaluate(TextSearcher searcher, String text, String pattern) {
    final int REPEAT = 100;
    Result result = new Result();
    result.start = System.currentTimeMillis();
    for (int i = 0; i < REPEAT; i++) {
      result.cnt = searcher.countOccurences(text, pattern, false, true);
    }
    result.end = System.currentTimeMillis();
    result.time = (result.end - result.start);
    return result;
  }
}
